﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Threading;
using System.ComponentModel;
using Benday.SynchronousSilverlightWcf.Business.TimeService;

namespace Benday.SynchronousSilverlightWcf.Business
{
    public class TimeFacade
    {
        private AutoResetEvent m_autoResetEvent = new AutoResetEvent(false);

        public void UpdateTimeInfo(TimeInformationViewModel viewModel)
        {
            if (viewModel == null)
            {
                throw new ArgumentNullException("viewModel", "Argument cannot be null.");
            }

            TimeService.TimeServiceClient client = new TimeService.TimeServiceClient();

            client.WhatTimeIsItCompleted += new EventHandler<TimeService.WhatTimeIsItCompletedEventArgs>(client_WhatTimeIsItCompleted);
            client.IsOddOrEvenCompleted += new EventHandler<TimeService.IsOddOrEvenCompletedEventArgs>(client_IsOddOrEvenCompleted);

            // call up to the server to get the current time
            var whatTimeIsItStatus = new AsyncCallStatus<TimeService.WhatTimeIsItCompletedEventArgs>();

            client.WhatTimeIsItAsync(whatTimeIsItStatus);
            m_autoResetEvent.WaitOne();

            if (whatTimeIsItStatus.CompletedEventArgs.Error != null)
            {
                throw whatTimeIsItStatus.CompletedEventArgs.Error;
            }

            DateTime currentTime = whatTimeIsItStatus.CompletedEventArgs.Result;

            // call the server to see if the tick value is odd or even
            var isOddOrEvenStatus = new AsyncCallStatus<TimeService.IsOddOrEvenCompletedEventArgs>();

            long timeAsTicks = currentTime.Ticks;

            client.IsOddOrEvenAsync(timeAsTicks, isOddOrEvenStatus);
            m_autoResetEvent.WaitOne();

            if (isOddOrEvenStatus.CompletedEventArgs.Error != null)
            {
                throw isOddOrEvenStatus.CompletedEventArgs.Error;
            }

            viewModel.CurrentTime = currentTime;
            viewModel.Ticks = timeAsTicks;
            viewModel.IsOddOrEven = isOddOrEvenStatus.CompletedEventArgs.Result;
        }

        void client_IsOddOrEvenCompleted(object sender, TimeService.IsOddOrEvenCompletedEventArgs e)
        {
            var status = e.UserState as AsyncCallStatus<IsOddOrEvenCompletedEventArgs>;

            status.CompletedEventArgs = e;

            m_autoResetEvent.Set();
        }

        void client_WhatTimeIsItCompleted(object sender, TimeService.WhatTimeIsItCompletedEventArgs e)
        {
            var status = e.UserState as AsyncCallStatus<WhatTimeIsItCompletedEventArgs>;

            status.CompletedEventArgs = e;

            m_autoResetEvent.Set();
        }
    }
}